@@ -43,14 +43,13 @@ enum connman_device_type {
 	CONNMAN_DEVICE_TYPE_BLUETOOTH = 4,
 	CONNMAN_DEVICE_TYPE_CELLULAR  = 5,
 	CONNMAN_DEVICE_TYPE_GPS       = 6,
+	CONNMAN_DEVICE_TYPE_GADGET    = 7,
 	CONNMAN_DEVICE_TYPE_VENDOR    = 10000,
 };
 
-enum connman_device_mode {
-	CONNMAN_DEVICE_MODE_UNKNOWN          = 0,
-	CONNMAN_DEVICE_MODE_NETWORK_SINGLE   = 1,
-	CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE = 2,
-};
+#define CONNMAN_DEVICE_PRIORITY_LOW      -100
+#define CONNMAN_DEVICE_PRIORITY_DEFAULT     0
+#define CONNMAN_DEVICE_PRIORITY_HIGH      100
 
 struct connman_device;
 
@@ -60,26 +59,20 @@ struct connman_device *connman_device_ref(struct connman_device *device);
 void connman_device_unref(struct connman_device *device);
 
 enum connman_device_type connman_device_get_type(struct connman_device *device);
-const char *connman_device_get_name(struct connman_device *device);
-const char *connman_device_get_path(struct connman_device *device);
 void connman_device_set_index(struct connman_device *device, int index);
 int connman_device_get_index(struct connman_device *device);
 void connman_device_set_interface(struct connman_device *device,
-				const char *interface, const char *control);
-const char *connman_device_get_control(struct connman_device *device);
+						const char *interface);
 
 void connman_device_set_ident(struct connman_device *device,
 						const char *ident);
 const char *connman_device_get_ident(struct connman_device *device);
 
-void connman_device_set_mode(struct connman_device *device,
-					enum connman_device_mode mode);
-enum connman_device_mode connman_device_get_mode(struct connman_device *device);
-
 int connman_device_set_powered(struct connman_device *device,
 						connman_bool_t powered);
 int connman_device_set_scanning(struct connman_device *device,
 						connman_bool_t scanning);
+void connman_device_reset_scanning(struct connman_device *device);
 
 int connman_device_set_disconnected(struct connman_device *device,
 						connman_bool_t disconnected);
