@@ -597,6 +597,8 @@ struct connman_element *connman_element_create(const char *name)
 
 	__connman_element_initialize(element);
 
+	element->name = g_strdup(name);
+
 	return element;
 }
 
@@ -1463,6 +1465,8 @@ void __connman_element_cleanup(void)
 	g_node_traverse(element_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 							free_node, NULL);
 
+	connman_element_unref(element_root->data);
+
 	g_node_destroy(element_root);
 	element_root = NULL;
 
