@@ -182,8 +182,7 @@ void connman_location_report_result(struct connman_location *location,
 	case CONNMAN_LOCATION_RESULT_UNKNOWN:
 		return;
 	case CONNMAN_LOCATION_RESULT_PORTAL:
-		__connman_service_indicate_state(location->service,
-						CONNMAN_SERVICE_STATE_LOGIN);
+		__connman_service_request_login(location->service);
 		break;
 	case CONNMAN_LOCATION_RESULT_ONLINE:
 		__connman_service_indicate_state(location->service,
@@ -226,8 +225,15 @@ int __connman_location_detect(struct connman_service *service)
 	if (location == NULL)
 		return -EINVAL;
 
-	if (location->driver)
-		return -EBUSY;
+	if (location->driver) {
+		location->result = CONNMAN_LOCATION_RESULT_UNKNOWN;
+		location->driver->finish(location);
+
+		if (location->driver->detect(location) == 0)
+			return 0;
+
+		location->driver = NULL;
+	}
 
 	for (list = driver_list; list; list = list->next) {
 		struct connman_location_driver *driver = list->data;
@@ -269,9 +275,12 @@ int __connman_location_finish(struct connman_service *service)
 
 int __connman_location_init(void)
 {
+	DBG("");
+
 	return 0;
 }
 
 void __connman_location_cleanup(void)
 {
+	DBG("");
 }
